///////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2004 Intel Corporation 
// All rights reserved. 
//
// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions are met: 
//
// * Redistributions of source code must retain the above copyright notice, 
// this list of conditions and the following disclaimer. 
// * Redistributions in binary form must reproduce the above copyright notice, 
// this list of conditions and the following disclaimer in the documentation 
// and/or other materials provided with the distribution. 
// * Neither name of Intel Corporation nor the names of its contributors 
// may be used to endorse or promote products derived from this software 
// without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL INTEL OR 
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
// OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
///////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ipp.h"
#include "tools.h"
#include "wave.h"

// Create an IIR filter that adds an echo in an IIRState
IppStatus myEchoInitAlloc_32f(IppsIIRState_32f** pState, int len, int delay)
{
	int tapsLen = 2*(len+1);
	Ipp32f* pTaps = ippsMalloc_32f(tapsLen);
	Ipp32f* pATaps = pTaps + len+1;
	ippsZero_32f(pTaps, tapsLen);

	pTaps[0] = 1.0f;
	pATaps[0] = 1.0f;
	if ((delay <= len) && (delay > 0)) pATaps[delay-1]=0.3f;
	return ippsIIRInitAlloc_32f(pState, pTaps, len, 0);
}

int main(int argc, char* argv[])
{
	IppsIIRState_32f* pState;

	int len, filterLen;
	Ipp32f *pSignal, delay = 0.1;

	WaveFile wf;
	if (argc<1)
	{
		printf("Usage: IIR.exe <wavefile>\n");
		return -1;
	}
	if (wf.Open(argv[1]))
	{
		printf("File %s not found\n", argv[1]);
		return -1;
	}

	WaveMem wm;
	wm.ReadFromFile(&wf);
	wm.Play(0);

	len = wm.GetLen();
	pSignal = ippsMalloc_32f(len);
	ippsConvert_16s32f(wm.GetShortBuffer(), pSignal, len);

	spView_32f(pSignal+8000, 8000, "Source", 0);

	filterLen = (int) wf.GetSampleRate() * delay;
	myEchoInitAlloc_32f(&pState, filterLen, (int)wf.GetSampleRate() * delay);

	IppStatus st = ippsIIR_32f_I(pSignal, len, pState);
	ippsConvert_32f16s_Sfs(pSignal, wm.GetShortBuffer(), len, ippRndNear, 0);

	wm.Play(0);

	spView_32f(pSignal+8000, 8000, "Filtered", 1);
	
	ippsFree(pSignal);

	return 0;
}
